/* ULTRIX compiling : cc syntaxe.c -DULTRIX -lcurses -ltermcap */

#define HELP "\
********************************************************************************\
********* Program  pro  syntaktickou  kontrolu  aritmetickych vyrazu, **********\
********** sestavenych ze scitani, odcitani, nasobeni a deleni cisel ***********\
********* ci alfanumerickych promennych s obecnym vnorovanim zavorek. **********\
********************************************************************************"

#ifdef ULTRIX
#include <cursesX.h>
#else
#include <conio.h>
#endif

#include <ctype.h>
#include <stdio.h>

int formula[80],           /* retezec vstupnich hodnot ASCII znaku vyrazu */
    len,                   /* delka vyrazu */
    syntax_error;          /* pocet nalezenych syntaktickych chyb */

void input_formul(void);        /* funkce pro nacteni vyrazu */
void syntax_test(void);         /* funkce provadejici syntakt. kontrolu */
void not_minus(int backchar);   /* test - pokud je nalezen znak (+,*,/) */
void defect(int);               /* zpracovani nalezene chyby */
int  is_signum(int letter);     /* testuje jestli znak je znamenko */


int main(void)
{
#ifdef ULTRIX
	initscr ();
	clear();
	refresh();
#else
	clrscr();
#endif

	printf("%sZADEJ VYRAZ :\n", HELP);

	input_formul();                  /* volani funkce nacteni vyrazu */

	syntax_test();                   /* testovani vyrazu */

	if( !syntax_error )                              /* test celkove      */
		printf("\nVyraz je syntakticky spravny !"); /* spravnosti vyrazu */

	getch();                         /* cekani na stisk klavesy */
	printf("\n");

return(0);
}


void input_formul(void)                   /* funkce pro nacteni vyrazu */
{
	int key;                          /* hodnota nacteneho ASCII znaku */

	for(len = 0; len <= 79;) {        /* test stisku ENTER */
                key = getch();
		if( key == 13 )
			break;
		if( key != 8 ) {                      /* test stisku BACKSPACE */
#ifndef ULTRIX
			printf("%c", key);
#endif
			formula[len++] = key;            /* ulozeni nacteneho znaku */
			}
		else {
			if( len > 0 ) {             /* pokud kurzor neni na prvnim */
#ifndef ULTRIX
				printf("\b");
#endif
				printf(" \b");
				--len;                 /* kurzorem bude smazan        */
				}
			}
	}
	formula[len] = '\0';                  /* ukonceni retezce */

return;
}


void syntax_test(void)              /* syntakticky test vyrazu */
{
	int i,                         /* pozice ve vyrazu */
	    backchar = ' ',            /* posledni znak ktery neni mezerou */
	    lastchar = ' ',            /* jakykoliv posledne zparacovany znak */
	    bracket_l = 0,             /* pocet nalezenych levych zavorek */
	    bracket_r = 0;             /* pocet nalezenych pravych zavorek */

	for(i = 0; i < len; ++i) {     /* uplny popis funkce je uveden */
							 /* v programatorske dokumentaci */
		switch (formula[i]) {

			case ' ':
				break;
			case '.':
				if( backchar == ')' || isalnum(backchar)
				    && lastchar == ' ' )
					defect(8);
				break;
			case '-':
				if( is_signum(backchar) ) {
					defect(2);
					}
				else
					if( backchar == '.' )
						defect(8);
				break;
			case '+':
				not_minus(backchar);
				break;
			case '*':
				not_minus(backchar);
				break;
			case '/':
				not_minus(backchar);
				break;
			case '(':
				++bracket_l;
				if( isalnum(backchar) || backchar == ')' )
					defect(3);
				break;
			case ')':
				++bracket_r;
				if( is_signum(backchar) || backchar == '(' )
					defect(4);
				break;

			default:
				if( isalpha(formula[i]) ) {
					if( backchar == ')' || isdigit(lastchar) ||
						lastchar == ' ' && isalnum(backchar) )
						defect(1);
					if( lastchar == '.' )
						defect(8);
					}
				else
					if( isdigit(formula[i]) ) {
						if( backchar == ')' || lastchar == ' '
							&& isalnum(backchar) )
						defect(8);
						}
					else
						defect(5);
		}
		lastchar = formula[i];
		if ( lastchar != ' ' )
			backchar = lastchar;
	}

	if( bracket_l != bracket_r )
		defect(6);
	if( backchar != ')' && !isalnum(backchar) )
		defect(7);

return;
}


void not_minus(int backchar)     /* kontrola pokud je znak  + , * , /  */
{
	if( backchar == '(' || backchar == ' ' ||
	    is_signum(backchar) || backchar == '.' )
		defect(2);
return;
}


void defect(int level)         /* zpracovani chyboveho vystupu */
{
	switch (level) {
		case 1:
			printf("\nPred promennou neni znamenko !");
			break;
		case 2:
			printf("\nChybne umistene znamenko !");
			break;
		case 3:
			printf("\nPred levou zavorkou chybi znamenko !");
			break;
		case 4:
			printf("\nPred pravou zavorkou je nespravny znak !");
			break;
		case 5:
			printf("\nNespravny znak !");
			break;
		case 6:
			printf("\nPocet levych a pravych zavorek nesouhlasi !");
			break;
		case 7:
			printf("\nVyraz je nespravne ukoncen !");
			break;
		case 8:
			printf("\nChybne umistena desitenna tecka !");
	}
	++syntax_error;              /* inkrementace poctu nalezenych chyb */
return;
}


int is_signum(int letter)               /* test zda znak je znamenko    */
{                                       /* Vraci nulu pokud neni, jinak */
	switch (letter) {                  /* vrati ASCII jeho hodnotu   . */
		case '+': return('+');
		case '-': return('-');
		case '*': return('*');
		case '/': return('/');
		}
return(0);
}


